@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -144,6 +143,7 @@ out:
 		ofono_gprs_context_set_ipv4_address(gc, ip, TRUE);
 		ofono_gprs_context_set_ipv4_netmask(gc, STATIC_IP_NETMASK);
 		ofono_gprs_context_set_ipv4_dns_servers(gc, dns);
+		ofono_gprs_context_set_ipv4_gateway(gc, gateway);
 	} else
 		ofono_gprs_context_set_ipv4_address(gc, NULL, FALSE);
 
@@ -510,7 +510,7 @@ static void mbm_gprs_context_remove(struct ofono_gprs_context *gc)
 	g_free(gcd);
 }
 
-static struct ofono_gprs_context_driver driver = {
+static const struct ofono_gprs_context_driver driver = {
 	.name			= "mbmmodem",
 	.probe			= mbm_gprs_context_probe,
 	.remove			= mbm_gprs_context_remove,
